@@ -2,12 +2,12 @@
 use strict;
 $| = 1;
 
-# $Id: archiverpt.pl 637 2004-03-03 17:52:55Z abeltje $
+# $Id: archiverpt.pl 1155 2008-01-03 13:32:28Z abeltje $
 use vars qw( $VERSION );
-$VERSION = '0.003';
+$VERSION = '0.005';
 
 use Cwd;
-use File::Spec;
+use File::Spec::Functions;
 use File::Path;
 use File::Copy;
 use FindBin;
@@ -118,14 +118,31 @@ $opt{adir} && -d $opt{adir} or do {
 my $patch_level = get_patch( $opt{ddir} );
 $patch_level =~ tr/ //sd;
 
-my $archived_rpt = "rpt${patch_level}.rpt";
+SKIP_RPT: {
+    my $archived_rpt = catfile( $opt{adir}, "rpt${patch_level}.rpt" );
+    # Do not archive if it is already done
+    last SKIP_RPT
+        if -f $archived_rpt && !$opt{force};
 
-# Do not archive if it is already done
--f File::Spec->catfile( $opt{adir}, $archived_rpt ) && $opt{force} and exit;
+    my $mktest_rpt = catfile( $opt{ddir}, 'mktest.rpt' );
+    if ( -f $mktest_rpt ) {
+        copy( $mktest_rpt, $archived_rpt ) or
+            die "Cannot copy to '$archived_rpt': $!";
+    }
+}
 
-copy( File::Spec->catfile( $opt{ddir}, 'mktest.rpt' ), 
-      File::Spec->catfile( $opt{adir}, $archived_rpt ) ) or
-    die "Cannot copy to '$archived_rpt': $!";
+SKIP_OUT: {
+    my $archived_out = catfile( $opt{adir}, "out${patch_level}.out" );
+    # Do not archive if it is already done
+    last SKIP_OUT
+        if -f $archived_out && !$opt{force};
+
+    my $mktest_out = catfile( $opt{ddir}, 'mktest.out' );
+    if ( -f $mktest_out ) {
+        copy( $mktest_out, $archived_out ) or
+            die "Cannot copy to '$archived_out': $!";
+    }
+}
 
 SKIP_LOG: {
     my $archived_log = "log${patch_level}.log";
